@@ -133,6 +133,24 @@ namespace Banshee
 			string data = Resource.GetFileContents("Tables.sql");
 			ArrayList instructions = ParseRawSql(data);
 			ExecuteSqlStatements(instructions);
+			
+			try {
+			   Execute("PRAGMA synchronous = OFF");
+			} catch(ApplicationException) {
+			   DebugLog.Add("Could not set sqlite3 PRAGMA synchronous = OFF");
+			}
+			
+			try {
+			   QuerySingle("SELECT LastPlayedStamp FROM Tracks LIMIT 1");
+			} catch(ApplicationException) {
+			   Execute("ALTER TABLE Tracks ADD LastPlayedStamp INTEGER");
+			}
+			
+			try {
+			   QuerySingle("SELECT DateAddedStamp FROM Tracks LIMIT 1");
+			} catch(ApplicationException) {
+			   Execute("ALTER TABLE Tracks ADD DateAddedStamp INTEGER");
+			}
 		}
 		
 		private ArrayList ParseRawSql(string rawData)
