@@ -41,7 +41,8 @@ namespace Banshee
 	public class FileEncodeCompleteArgs : EventArgs
 	{
 		public TrackInfo Track;
-		public string EncodedFilePath;
+		public Uri InputUri;
+		public Uri EncodedFileUri;
 	}
 
 	public class FileEncodeTransaction : LibraryTransaction
@@ -68,9 +69,14 @@ namespace Banshee
 			statusMessage = Catalog.GetString("Initializing Encoder...");
 		}
 		
-		public void AddTrack(TrackInfo track, string outputFile)
+		public void AddTrack(TrackInfo track, Uri outputUri)
 		{
-			tracks[track] = outputFile;
+			tracks[track] = outputUri;
+		}
+		
+		public void AddTrack(Uri inputUri, Uri outputUri)
+		{
+		    tracks[inputUri] = outputUri;
 		}
 		
 		public override void Run()
@@ -78,31 +84,43 @@ namespace Banshee
 			encoder = new GstFileEncoder();
 			encoder.Progress += OnEncoderProgress;
 			
-			foreach(TrackInfo ti in tracks.Keys) {
-			    string outputFile = tracks[ti] as string;
+			foreach(object obj in tracks.Keys) {
+			    Uri outputUri = tracks[obj] as Uri;
+			    Uri inputUri = null;
 			    
-				statusMessage = String.Format(
-					Catalog.GetString("Encoding {0} - {1} ..."),
-					ti.Artist, ti.Title);
+			    if(obj is TrackInfo) {
+        				statusMessage = String.Format(
+        					Catalog.GetString("Encoding {0} - {1} ..."),
+        					(obj as TrackInfo).Artist, (obj as TrackInfo).Title);
+        			    inputUri = (obj as TrackInfo).Uri;
+        		    } else if(obj is Uri) {
+        		        statusMessage = Catalog.GetString("Encoding files...");
+        		        inputUri = obj as Uri;
+        		    }
 				
 				if(cancelRequested)
 					break;
 					
 				try {
-					string encPath = encoder.Encode(ti.Uri, outputFile, 
-					   profile);
-				
+					Uri encUri = encoder.Encode(inputUri, outputUri, profile);
+					   
 					FileEncodeCompleteHandler handler = FileEncodeComplete;
 					if(handler != null) {
 						FileEncodeCompleteArgs args = 
 							new FileEncodeCompleteArgs();
-						args.Track = ti;
-						args.EncodedFilePath = encPath;
+						if(obj is TrackInfo) {
+						  args.Track = obj as TrackInfo;
+						  args.InputUri = (obj as TrackInfo).Uri;
+						} else if(obj is Uri) {
+						  args.InputUri = (obj as Uri);
+						}
+						
+						args.EncodedFileUri = encUri;
 						handler(this, args);
 					}
 				} catch(Exception e) {
 					Console.WriteLine("Could not encode '{0}': {1}",
-						ti.Uri, e.Message);
+						inputUri.AbsoluteUri, e.Message);
 				}
 					
 					
