@@ -35,6 +35,7 @@ using Mono.Unix;
 using Gtk;
 using Gdk;
 using Glade;
+using System.IO;
 
 using Sql;
 
@@ -51,6 +52,7 @@ namespace Banshee
         [Widget] private Gtk.Image ImageNext;
         [Widget] private Gtk.Image ImagePlayPause;
         [Widget] private Gtk.Image ImageBurn;
+        [Widget] private Gtk.Image ImageRip;
         [Widget] private Gtk.HScale ScaleTime;
         [Widget] private Gtk.Label LabelInfo;
         [Widget] private Gtk.Label LabelStatusBar;
@@ -74,6 +76,7 @@ namespace Banshee
         private ProgressBar ipodDiskUsageBar;
         private Viewport sourceViewLoadingVP;
         private Button ipodSyncButton;
+        private CoverArtThumbnail cover_art;
         
         private bool incrementedCurrentSongPlayCount;
     
@@ -153,7 +156,8 @@ namespace Banshee
             
             Core.Log.Updated += OnLogCoreUpdated;
             
-            GLib.Timeout.Add(500, InitialLoadTimeout);
+            InitialLoadTimeout();
+            //GLib.Timeout.Add(500, InitialLoadTimeout);
     
             Gtk.Application.Run();
         }
@@ -169,6 +173,10 @@ namespace Banshee
                  CheckIpodForNew(device);
             }
             
+            if(Core.ArgumentQueue.Contains("audio-cd")) {
+                SelectAudioCd(Core.ArgumentQueue["audio-cd"]);
+            }
+            
             return false;
         }
           
@@ -211,6 +219,7 @@ namespace Banshee
             ImagePlayPause.SetFromStock("media-play", IconSize.LargeToolbar);
             
             ImageBurn.SetFromStock("media-burn", IconSize.LargeToolbar);
+            ImageRip.SetFromStock("media-rip", IconSize.LargeToolbar);
 
             gxml["ButtonBurn"].Visible = true;
                 
@@ -242,6 +251,11 @@ namespace Banshee
             volumeButton.VolumeChanged += 
                 new VolumeButton.VolumeChangedHandler(OnVolumeScaleChanged);
 
+            // Cover Art Thumbnail
+            cover_art = new CoverArtThumbnail(36);
+            (gxml["CoverArtContainer"] as Container).Add(cover_art);
+            cover_art.Hide();
+
             // Source View
             Label sourceViewLoading = new Label();
             sourceViewLoading.Yalign = 0.15f;
@@ -321,6 +335,8 @@ namespace Banshee
             box.PackStart(ipodEjectButton, false, false, 0);
             ipodEjectButton.ShowAll();
             
+            (gxml["LeftContainer"] as VBox).PackStart(new ActiveUserEventsManager(), false, false, 0);
+            
            // gxml["IpodContainer"].Visible = false;
             
             // Misc
@@ -420,10 +436,8 @@ namespace Banshee
         
         private void PromptForImport()
         {
-            Application.Invoke(delegate {
-            
-            HigMessageDialog md = new HigMessageDialog(WindowPlayer, 
-                DialogFlags.DestroyWithParent, MessageType.Question,
+           HigMessageDialog md = new HigMessageDialog(WindowPlayer, 
+                DialogFlags.Modal, MessageType.Question,
                 Catalog.GetString("Import Music"),
                 Catalog.GetString("Your music library is empty. You may import new music into " +
                 "your library now, or choose to do so later.\n\nAutomatic import " +
@@ -431,21 +445,24 @@ namespace Banshee
                 "be patient."),
                 Catalog.GetString("Import Folder"));
                 
-            md.AddButton(Catalog.GetString("Automatic Import"), 
-                Gtk.ResponseType.Apply, true);
+            md.AddButton(Catalog.GetString("Automatic Import"), ResponseType.Apply, true);
+            md.Response += OnPromptForImportResponse;
+            md.ShowAll();
+        }
+        
+        private void OnPromptForImportResponse(object o, ResponseArgs args)
+        {
+            (o as Dialog).Response -= OnPromptForImportResponse;
+            (o as Dialog).Destroy();
             
-            switch(md.Run()) {
-                case (int)ResponseType.Ok:
+            switch(args.ResponseId) {
+                case ResponseType.Ok:
                     ImportWithFileSelector();
                     break;
-                case (int)ResponseType.Apply:
+                case ResponseType.Apply:
                     ImportHomeDirectory();
                     break;
             }
-            
-            md.Destroy();
-            
-            });
         }
         
         private void ConnectToLibraryTransactionManager()
@@ -510,10 +527,12 @@ namespace Banshee
         
         private void OnLibraryReloaded(object o, EventArgs args)
         {
+            startupLoadReady = true;
+            
             if(Core.Library.Tracks.Count <= 0) {
-                GLib.Timeout.Add(500, PromptForImportTimeout);
-            } else {
-                startupLoadReady = true;
+                Application.Invoke(delegate { 
+                    PromptForImport();
+                });
             }
         }
         
@@ -529,6 +548,7 @@ namespace Banshee
       
         private void Quit()
         {
+            ActiveUserEventsManager.Instance.CancelAll();
             playlistView.Shutdown();
             Core.Instance.Player.Dispose();
             Core.GconfClient.Set(GConfKeys.SourceViewWidth, SourceSplitter.Position);
@@ -564,6 +584,8 @@ namespace Banshee
             trackInfoHeader.Artist = ti.DisplayArtist;
             trackInfoHeader.Title = ti.DisplayTitle;
             
+            cover_art.Track = ti;
+            
             if(trayIcon != null) {
                 trayIcon.Tooltip = ti.DisplayArtist + " - " + ti.DisplayTitle;
             }
@@ -623,15 +645,38 @@ namespace Banshee
             Core.GconfClient.Set(GConfKeys.WindowHeight, height);
         }
         
+        [GLib.ConnectBefore]
         private void OnKeyPressEvent(object o, KeyPressEventArgs args)
         {
+            bool handled = false;
+            
             switch(args.Event.Key) {
                 case Gdk.Key.J:
                 case Gdk.Key.j:
                 case Gdk.Key.F3:
-                    searchEntry.HasFocus = true;
+                    if(!searchEntry.HasFocus) {
+                        searchEntry.Focus();
+                        handled = true;
+                    }
+                    break;
+                case Gdk.Key.Left:
+                    if(args.Event.State == Gdk.ModifierType.ControlMask) {
+                        Core.Instance.Player.Position -= 10;
+                        handled = true;
+                    } else if(args.Event.State == Gdk.ModifierType.ShiftMask) {
+                        Core.Instance.Player.Position = 0;
+                        handled = true;
+                    }
+                    break;
+                case Gdk.Key.Right:
+                    if(args.Event.State == Gdk.ModifierType.ControlMask) {
+                        Core.Instance.Player.Position += 10;
+                        handled = true;
+                    } 
                     break;
             }
+            
+            args.RetVal = handled;
         }
               
         // ---- Tray Event Handlers ----
@@ -733,6 +778,18 @@ namespace Banshee
         {
             new PreferencesWindow();
         }
+        
+        private bool is_fullscreen = false;
+        private void OnMenuFullScreenActivate(object o, EventArgs args)
+        {
+            if(is_fullscreen) {
+                WindowPlayer.Unfullscreen();
+                is_fullscreen = false;
+            } else {
+                WindowPlayer.Fullscreen();
+                is_fullscreen = true;
+            }
+        }
 
         // ---- Player Event Handlers ----
         
@@ -1020,6 +1077,7 @@ namespace Banshee
                  || source.Type == SourceType.Library;
             searchEntry.Sensitive = gxml["SearchLabel"].Sensitive;
             playlistView.SyncColumn.Visible = source.Type == SourceType.Ipod;
+            playlistView.RipColumn.Visible = source.Type == SourceType.AudioCd;
             
             playlistModel.ImportCanUpdate = source.Type == SourceType.Library
                 && searchEntry.Query == String.Empty;
@@ -1055,24 +1113,23 @@ namespace Banshee
         
         private void OnIpodDeviceChanged(object o, EventArgs args)
         {
-            IPod.Device device = o as IPod.Device;
-            
-            foreach(object [] obj in (sourceView.Model as ListStore)) {
-                if(obj[0] is IpodSource && (obj[0] as IpodSource).Device == device) {
-                    Application.Invoke(delegate {
-                        (obj[0] as IpodSource).SetSourceName(device.Name);
-                        sourceView.QueueDraw();
-                    });
-                }
-            }
-            
-            if(playlistModel.Source is IpodSource && 
-                (playlistModel.Source as IpodSource).Device == device) {
-                UpdateIpodDiskUsageBar(playlistModel.Source as IpodSource);
-                (gxml["ViewNameLabel"] as Label).Markup = "<b>" 
-                    + GLib.Markup.EscapeText(device.Name) + "</b>";
-                 sourceView.QueueDraw();
-            }
+			Application.Invoke (delegate {
+				IPod.Device device = o as IPod.Device;
+				
+				foreach(object [] obj in (sourceView.Model as ListStore)) {
+					if(obj[0] is IpodSource && (obj[0] as IpodSource).Device == device) {
+						(obj[0] as IpodSource).SetSourceName(device.Name);
+						sourceView.QueueDraw();
+					}
+				}
+				
+				if(playlistModel.Source is IpodSource && (playlistModel.Source as IpodSource).Device == device) {
+					UpdateIpodDiskUsageBar(playlistModel.Source as IpodSource);
+					(gxml["ViewNameLabel"] as Label).Markup = "<b>" 
+						+ GLib.Markup.EscapeText(device.Name) + "</b>";
+					sourceView.QueueDraw();
+				}
+			});
         }
         
         private void OnIpodCoreDeviceAdded(object o, IpodDeviceArgs args)
@@ -1206,7 +1263,6 @@ namespace Banshee
         {
             if(playlistModel.Source.Type == SourceType.Ipod 
                 && (playlistModel.Source as IpodSource).IsSyncing) {
-                playlistModel.ClearModel();
                 playlistView.Sensitive = false;
             }
         }
@@ -1336,26 +1392,50 @@ namespace Banshee
         }
         
         private void OnItemRemoveActivate(object o, EventArgs args)
+        {
+            RemoveSongs(false);
+        }
+        
+        private void OnItemRemoveFileSystemActivate(object o, EventArgs args)
+        {
+            RemoveSongs(true);
+        }
+             
+        private void RemoveSongs(bool deleteFromFileSystem)
         {
             // Don't steal "Del" key from the search entry
             if (WindowPlayer.Focus is Entry &&
                 Gtk.Global.CurrentEvent is Gdk.EventKey) {
-                Gtk.Bindings.ActivateEvent(WindowPlayer.Focus, (Gdk.EventKey)Gtk.Global.CurrentEvent);
+                Gtk.Bindings.ActivateEvent(WindowPlayer.Focus, 
+                    (Gdk.EventKey)Gtk.Global.CurrentEvent);
                 return;
             }
 
             int selCount = playlistView.Selection.CountSelectedRows();
         
-            if(selCount <= 0)
+            if(selCount <= 0) {
                 return;
-        
+            }
+            
             if(playlistModel.Source.Type == SourceType.Library) {
-                string msg = String.Format(
+                string msg = String.Empty;
+                
+                if(deleteFromFileSystem) {
+                    msg = String.Format(
+                    Catalog.GetPluralString(
+                        "Are you sure you want to remove the selected song from your library <i><b>and</b></i> your drive? This action will permanently delete the file.",
+                        "Are you sure you want to remove the selected <b>({0})</b> songs from your library <i><b>and</b></i> your drive? This action will permanently delete the files.",
+                        selCount),
+                    selCount);
+                } else {
+                    msg = String.Format(
                     Catalog.GetPluralString(
                         "Are you sure you want to remove the selected song from your library?",
                         "Are you sure you want to remove the selected <b>({0})</b> songs from your library?",
                         selCount),
                     selCount);
+                }
+                    
                 HigMessageDialog md = new HigMessageDialog(WindowPlayer, 
                     DialogFlags.DestroyWithParent, MessageType.Warning,
                     ButtonsType.YesNo,
@@ -1372,9 +1452,10 @@ namespace Banshee
             TreeIter [] iters = new TreeIter[selCount];
             int i = 0;
             
-            foreach(TreePath path in playlistView.Selection.GetSelectedRows())            
+            foreach(TreePath path in playlistView.Selection.GetSelectedRows()) {
                 playlistModel.GetIter(out iters[i++], path);
-        
+            }
+            
             TrackRemoveTransaction transaction;
             
             if(playlistModel.Source.Type == SourceType.Ipod) {
@@ -1387,16 +1468,34 @@ namespace Banshee
                 }
                 sourceView.QueueDraw();
                 return;
-            } else if(playlistModel.Source.Type == SourceType.Library)
+            } else if(playlistModel.Source.Type == SourceType.Library) {
                 transaction = new LibraryTrackRemoveTransaction();
-            else
+            } else {
                 transaction = new PlaylistTrackRemoveTransaction(
                     Playlist.GetId(playlistModel.Source.Name));
-                
+            }
+              
             for(i = 0; i < iters.Length; i++) {
                 TrackInfo ti = playlistModel.IterTrackInfo(iters[i]);
                 playlistModel.RemoveTrack(ref iters[i]);
                 transaction.RemoveQueue.Add(ti);
+                
+                if(deleteFromFileSystem) {
+                    try {
+                        File.Delete(ti.Uri.LocalPath);
+                    } catch(Exception) {
+                        Console.WriteLine("Could not delete file: " + ti.Uri.LocalPath);
+                    }
+                    
+                    // trim empty parent directories
+                    try {
+                        string old_dir = Path.GetDirectoryName(ti.Uri.LocalPath);
+                        while(old_dir != null && old_dir != String.Empty) {
+                            Directory.Delete(old_dir);
+                            old_dir = Path.GetDirectoryName(old_dir);
+                        }
+                    } catch(Exception) {}
+                }
             }
             
             transaction.Finished += OnLibraryTrackRemoveFinished;
@@ -1582,7 +1681,12 @@ namespace Banshee
         
         private void OnItemRemoveSongsActivate(object o, EventArgs args)
         {
-            OnItemRemoveActivate(o, args);
+            RemoveSongs(false);
+        }
+        
+        private void OnItemDeleteSongsFileSystemActivate(object o, EventArgs args)
+        {
+            RemoveSongs(true);
         }
         
         private void OnItemDeletePlaylistActivate(object o, EventArgs args)
@@ -2002,13 +2106,15 @@ namespace Banshee
         
         private void OnButtonBurnClicked(object o, EventArgs args)
         {
-            if(playlistView.Selection.CountSelectedRows() <= 0)
+            if(playlistView.Selection.CountSelectedRows() <= 0) {
                 return;
+            }
         
             BurnCore burnCore = new BurnCore(BurnCore.DiskType.Audio);
         
-            foreach(TreePath path in playlistView.Selection.GetSelectedRows())
+            foreach(TreePath path in playlistView.Selection.GetSelectedRows()) {
                 burnCore.AddTrack(playlistModel.PathTrackInfo(path));
+            }
                 
             burnCore.Burn();
         }
@@ -2018,12 +2124,22 @@ namespace Banshee
             RipTransaction trans = new RipTransaction();
 
             foreach(object [] node in playlistModel) {
-                if(node[0].GetType() == typeof(AudioCdTrackInfo))
+                if(node[0] is AudioCdTrackInfo && ((AudioCdTrackInfo)node[0]).CanRip) {
                     trans.QueueTrack(node[0] as AudioCdTrackInfo);
+                }
             }
             
-            if(trans.QueueSize > 0)
-                trans.Register();
+            if(trans.QueueSize > 0) {
+                trans.Run();
+            } else {
+                HigMessageDialog dialog = new HigMessageDialog(WindowPlayer, DialogFlags.Modal, 
+                    MessageType.Info, ButtonsType.Ok, 
+                    Catalog.GetString("Invalid Selection"),
+                    Catalog.GetString("You must select at least one track to import.")
+                );
+                dialog.Run();
+                dialog.Destroy();
+            }
         }
         
         private void EjectSource(Source source)
@@ -2031,8 +2147,7 @@ namespace Banshee
             if(source.CanEject) {
                 try {
                     if(source.GetType() == typeof(IpodSource)) {
-                        if(activeTrackInfo != null && activeTrackInfo.GetType() 
-                            == typeof(IpodTrackInfo)) {
+                        if(activeTrackInfo != null && activeTrackInfo is IpodTrackInfo) {
                             StopPlaying();
                         }
                     }
