@@ -31,7 +31,7 @@ using System;
 using Gtk;
 using Glade;
 using System.Collections;
-using Mono.Posix;
+using Mono.Unix;
 
 namespace Banshee
 {
@@ -42,7 +42,7 @@ namespace Banshee
 		public string Artist;
 		public string Album;
 		public string Title;
-		public string Uri;
+		public Uri Uri;
 	
 		public uint TrackNumber;
 		public uint TrackCount;
@@ -88,6 +88,7 @@ namespace Banshee
 		[Widget] private Button SaveButton;
 		[Widget] private Button Previous;
 		[Widget] private Button Next;
+		[Widget] private Button TrackNumberIterator;
 		[Widget] private Button TrackNumberSync;
 		[Widget] private Button TrackCountSync;
 		[Widget] private Button ArtistSync;
@@ -121,8 +122,7 @@ namespace Banshee
 			glade = new Glade.XML(null, 
 				"trackinfo.glade", "WindowTrackInfo", null);
 			glade.Autoconnect(this);
-			WindowTrackInfo.Icon = 
-				Gdk.Pixbuf.LoadFromResource("banshee-icon.png");
+			WindowTrackInfo.Icon = ThemeIcons.WindowManager;
 	
 			(glade["BackImage"] as Image).SetFromStock("gtk-go-back", 
 				IconSize.Button);
@@ -134,6 +134,7 @@ namespace Banshee
 			Previous.Clicked += OnPreviousClicked;
 			Next.Clicked += OnNextClicked;
 			
+			TrackNumberIterator.Clicked += OnTrackNumberIteratorClicked;
 			TrackNumberSync.Clicked += OnTrackNumberSyncClicked;
 			TrackCountSync.Clicked += OnTrackCountSyncClicked;
 			ArtistSync.Clicked += OnArtistSyncClicked;
@@ -144,6 +145,7 @@ namespace Banshee
 			Previous.Visible = TrackSet.Count > 1;
 				
 			//glade["MultiTrackHeader"].Visible = TrackSet.Count > 1;
+			TrackNumberIterator.Visible = TrackSet.Count > 1;
 			TrackNumberSync.Visible = TrackSet.Count > 1;
 			TrackCountSync.Visible = TrackSet.Count > 1;
 			ArtistSync.Visible = TrackSet.Count > 1;
@@ -153,6 +155,9 @@ namespace Banshee
 			tips.SetTip(TrackNumberSync, 
 				Catalog.GetString("Set all Track Numbers to this value"), 
 				"track numbers");
+			tips.SetTip(TrackNumberIterator, 
+				Catalog.GetString("Automatically Set All Track Numbers"), 
+				"track iterator");
 			tips.SetTip(TrackCountSync, 
 				Catalog.GetString("Set all Track Counts to this value"),
 				"track counts");
@@ -216,7 +221,7 @@ namespace Banshee
 		    
 		    tips.SetTip(glade["TitleEventBox"],
 				String.Format(Catalog.GetString("File: {0}"), 
-				    track.Uri), "uri");
+				    track.Uri.AbsoluteUri), "uri");
 		
 			Previous.Sensitive = index > 0;
 			Next.Sensitive = index < TrackSet.Count - 1;
@@ -240,6 +245,19 @@ namespace Banshee
 				track.TrackNumber = (uint)TrackNumber.Value;
 		}
 		
+		private void OnTrackNumberIteratorClicked(object o, EventArgs args)
+		{
+			int i = 1;
+			foreach(EditorTrack track in TrackSet) {
+				track.TrackNumber = (uint)i++;
+				track.TrackCount = (uint) TrackSet.Count;
+			}
+
+			EditorTrack current_track = TrackSet[currentIndex] as EditorTrack;
+			TrackNumber.Value = (int) current_track.TrackNumber;
+			TrackCount.Value = (int) current_track.TrackCount;
+		}
+		
 		private void OnTrackCountSyncClicked(object o, EventArgs args)
 		{
 			foreach(EditorTrack track in TrackSet)
